@@ -6,15 +6,18 @@
 
 from __future__ import print_function, division, absolute_import
 import os
+import sys
 import logging
 import json
 from typing import Callable, Dict, Optional, Union
 
-__version__ = ""
-with open(
-    os.path.join(os.path.dirname(os.path.abspath(__file__)), "ads_version.json")
-) as version_file:
-    __version__ = json.load(version_file)["version"]
+# https://packaging.python.org/en/latest/guides/single-sourcing-package-version/#single-sourcing-the-package-version
+if sys.version_info >= (3, 8):
+    from importlib import metadata
+else:
+    import importlib_metadata as metadata
+
+__version__ = metadata.version("oracle_ads")
 import oci
 
 import matplotlib.font_manager  # causes matplotlib to regenerate its fonts
@@ -122,13 +125,8 @@ def _set_test_mode(mode=False):
 
 
 def hello():
-    """
-    Imports Pandas, sets the documentation mode, and prints a fancy "Hello".
-    """
-    import pandas
-
-    global documentation_mode
-    global debug_mode
+    import oci
+    import ocifs
 
     print(
         f"""
@@ -139,9 +137,10 @@ o---o|   O o-o
 |   ||  /     |
 o   oo-o  o--o
 
-ADS SDK version: {__version__}
-Pandas version: {pandas.__version__}
-Debug mode: {debug_mode}
+ads v{__version__}
+oci v{oci.__version__}
+ocifs v{ocifs.__version__}
+
 """
     )
 
