@@ -5,7 +5,7 @@
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 
-class ChangesNotCommitted(Exception):
+class ChangesNotCommitted(Exception):   # pragma: no cover
     def __init__(self, path):
         msg = f"""
             File(s) at {path} are either dirty or untracked.
