@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2020, 2023 Oracle and/or its affiliates.
+# Copyright (c) 2020, 2024 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 import contextlib
@@ -14,7 +14,7 @@ from ads.common.config import DEFAULT_CONFIG_PATH, DEFAULT_CONFIG_PROFILE, Confi
 OCI_ODSC_SERVICE_ENDPOINT = os.environ.get("OCI_ODSC_SERVICE_ENDPOINT")
 OCI_IDENTITY_SERVICE_ENDPOINT = os.environ.get("OCI_IDENTITY_SERVICE_ENDPOINT")
 NB_SESSION_COMPARTMENT_OCID = os.environ.get("NB_SESSION_COMPARTMENT_OCID")
-PROJECT_OCID = os.environ.get("PROJECT_OCID")
+PROJECT_OCID = os.environ.get("PROJECT_OCID") or os.environ.get("PIPELINE_PROJECT_OCID")
 NB_SESSION_OCID = os.environ.get("NB_SESSION_OCID")
 USER_OCID = os.environ.get("USER_OCID")
 OCI_RESOURCE_PRINCIPAL_VERSION = os.environ.get("OCI_RESOURCE_PRINCIPAL_VERSION")
@@ -24,18 +24,67 @@ TENANCY_OCID = os.environ.get("TENANCY_OCID")
 OCI_REGION_METADATA = os.environ.get("OCI_REGION_METADATA")
 JOB_RUN_OCID = os.environ.get("JOB_RUN_OCID")
 JOB_RUN_COMPARTMENT_OCID = os.environ.get("JOB_RUN_COMPARTMENT_OCID")
+PIPELINE_RUN_OCID = os.environ.get("PIPELINE_RUN_OCID")
+PIPELINE_RUN_COMPARTMENT_OCID = os.environ.get("PIPELINE_RUN_COMPARTMENT_OCID")
+PIPELINE_COMPARTMENT_OCID = os.environ.get("PIPELINE_COMPARTMENT_OCID")
+
 CONDA_BUCKET_NAME = os.environ.get("CONDA_BUCKET_NAME", "service-conda-packs")
 CONDA_BUCKET_NS = os.environ.get("CONDA_BUCKET_NS", "id19sfcrra6z")
 OCI_RESOURCE_PRINCIPAL_RPT_ENDPOINT = os.environ.get(
     "OCI_RESOURCE_PRINCIPAL_RPT_ENDPOINT"
 )
-COMPARTMENT_OCID = NB_SESSION_COMPARTMENT_OCID or JOB_RUN_COMPARTMENT_OCID
+PROJECT_COMPARTMENT_OCID = os.environ.get("PROJECT_COMPARTMENT_OCID")
+COMPARTMENT_OCID = (
+    NB_SESSION_COMPARTMENT_OCID
+    or PROJECT_COMPARTMENT_OCID
+    or JOB_RUN_COMPARTMENT_OCID
+    or PIPELINE_RUN_COMPARTMENT_OCID
+)
 MD_OCID = os.environ.get("MD_OCID")
 DATAFLOW_RUN_OCID = os.environ.get("DATAFLOW_RUN_ID")
+
 RESOURCE_OCID = (
-    NB_SESSION_OCID or JOB_RUN_OCID or MD_OCID
-)  # We can add DATAFLOW_RUN_OCID here. Needs impact analysis
+    NB_SESSION_OCID or JOB_RUN_OCID or MD_OCID or PIPELINE_RUN_OCID or DATAFLOW_RUN_OCID
+)
 NO_CONTAINER = os.environ.get("NO_CONTAINER")
+TMPDIR = os.environ.get("TMPDIR")
+
+ODSC_MODEL_COMPARTMENT_OCID = os.environ.get("ODSC_MODEL_COMPARTMENT_OCID")
+AQUA_MODEL_DEPLOYMENT_IMAGE = os.environ.get("AQUA_MODEL_DEPLOYMENT_IMAGE")
+AQUA_MODEL_DEPLOYMENT_CONFIG = os.environ.get(
+    "AQUA_DEPLOYMENT_CONFIG", "deployment_config.json"
+)
+AQUA_MODEL_FINETUNING_CONFIG = os.environ.get(
+    "AQUA_MODEL_FINETUNING_CONFIG", "ft_config.json"
+)
+AQUA_CONTAINER_INDEX_CONFIG = os.environ.get(
+    "AQUA_CONTAINER_INDEX_CONFIG", "container_index.json"
+)
+AQUA_MODEL_DEPLOYMENT_CONFIG_DEFAULTS = os.environ.get(
+    "AQUA_MODEL_DEPLOYMENT_CONFIG_DEFAULTS", "deployment_config_defaults.json"
+)
+AQUA_RESOURCE_LIMIT_NAMES_CONFIG = os.environ.get(
+    "AQUA_RESOURCE_LIMIT_NAMES_CONFIG", "resource_limit_names.json"
+)
+AQUA_DEPLOYMENT_CONTAINER_METADATA_NAME = "deployment-container"
+AQUA_FINETUNING_CONTAINER_METADATA_NAME = "finetune-container"
+AQUA_EVALUATION_CONTAINER_METADATA_NAME = "evaluation-container"
+AQUA_MODEL_DEPLOYMENT_FOLDER = "/opt/ds/model/deployed_model/"
+AQUA_SERVED_MODEL_NAME = "odsc-llm"
+AQUA_CONFIG_FOLDER = os.path.join(
+    os.path.dirname(os.path.realpath(__file__)), "aqua/config/"
+)
+AQUA_JOB_SUBNET_ID = os.environ.get("AQUA_JOB_SUBNET_ID", None)
+AQUA_SERVICE_MODELS_BUCKET = os.environ.get(
+    "AQUA_SERVICE_MODELS_BUCKET", "service-managed-models"
+)
+AQUA_TELEMETRY_BUCKET = os.environ.get(
+    "AQUA_TELEMETRY_BUCKET", "service-managed-models"
+)
+AQUA_TELEMETRY_BUCKET_NS = os.environ.get("AQUA_TELEMETRY_BUCKET_NS", CONDA_BUCKET_NS)
+DEBUG_TELEMETRY = os.environ.get("DEBUG_TELEMETRY", None)
+AQUA_SERVICE_NAME = "aqua"
+DATA_SCIENCE_SERVICE_NAME = "data-science"
 
 
 def export(
