@@ -1,13 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2020, 2021 Oracle and its affiliates.
+# Copyright (c) 2020, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 from ads.explanations import MLXGlobalExplainer, MLXLocalExplainer
 from ads.explanations.mlx_whatif_explainer import MLXWhatIfExplainer
 from ads.explanations.mlx_interface import _reset_index
-from ads.common import utils
+from ads.common.decorator.deprecate import deprecated
 
 
 class ADSExplainer(object):
@@ -16,6 +16,10 @@ class ADSExplainer(object):
     explanation objects.
     """
 
+    @deprecated(
+        details="Working with AutoML has moved from within ADS to working directly with the AutoMLx library. AutoMLx are preinstalled in conda pack automlx_p38_cpu_v2 and later, and can now be updated independently of ADS. AutoMLx documentation may be found at https://docs.oracle.com/en-us/iaas/tools/automlx/latest/html/multiversion/v23.1.1/index.html. Notebook examples are in Oracle's samples repository: https://github.com/oracle-samples/oci-data-science-ai-samples/tree/master/notebook_examples and a migration tutorial can be found at https://accelerated-data-science.readthedocs.io/en/latest/user_guide/model_training/automl/quick_start.html .",
+        raise_error=True,
+    )
     def __init__(self, test_data, model, training_data=None):
         """
         Creates an ads explainer object.
