@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*--
 
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 """
@@ -262,5 +262,5 @@ class EDAMixin:
             warning_df = self._obj[col].ads.warning()
             if warning_df is not None:
                 warning_df.insert(0, "Column", col)
-                result_df = result_df.append(warning_df)
+                result_df = pd.concat([result_df, warning_df])
         return result_df.reset_index(drop=True)
