@@ -31,11 +31,25 @@ from ads.model.runtime.env_info import InferenceEnvInfo
 from oci.data_flow.models import CreateApplicationDetails, CreateRunDetails
 from tqdm import tqdm
 
+from ads.config import NB_SESSION_COMPARTMENT_OCID, NB_SESSION_OCID
+
 logger = logging.getLogger(__name__)
 
 CONDA_PACK_SUFFIX = "#conda"
 SLEEP_INTERVAL = 3
 
+DEFAULT_LANGUAGE = "PYTHON"
+DEFAULT_SPARK_VERSION = "3.2.1"
+DEFAULT_NUM_EXECUTORS = 1
+DEFAULT_SHAPE = "VM.Standard.E3.Flex"
+DATAFLOW_SHAPE_FAMILY = [
+    "Standard.E3",
+    "Standard.E4",
+    "Standard3",
+    "Standard.A1",
+    "Standard2",
+]
+
 
 def conda_pack_name_to_dataflow_config(conda_uri):
     return {
@@ -360,7 +374,6 @@ class DataFlowLogs:
 
 
 class DataFlow(Infrastructure):
-
     CONST_COMPARTMENT_ID = "compartment_id"
     CONST_CONFIG = "configuration"
     CONST_EXECUTE = "execute"
@@ -378,6 +391,9 @@ class DataFlow(Infrastructure):
     CONST_OCPUS = "ocpus"
     CONST_ID = "id"
     CONST_PRIVATE_ENDPOINT_ID = "private_endpoint_id"
+    CONST_POOL_ID = "pool_id"
+    CONST_FREEFORM_TAGS = "freeform_tags"
+    CONST_DEFINED_TAGS = "defined_tags"
 
     attribute_map = {
         CONST_COMPARTMENT_ID: "compartmentId",
@@ -396,6 +412,9 @@ class DataFlow(Infrastructure):
         CONST_OCPUS: CONST_OCPUS,
         CONST_ID: CONST_ID,
         CONST_PRIVATE_ENDPOINT_ID: "privateEndpointId",
+        CONST_POOL_ID: "poolId",
+        CONST_FREEFORM_TAGS: "freeformTags",
+        CONST_DEFINED_TAGS: "definedTags",
     }
 
     def __init__(self, spec: dict = None, **kwargs):
@@ -408,7 +427,11 @@ class DataFlow(Infrastructure):
             spec = {
                 k: v
                 for k, v in spec.items()
-                if f"with_{camel_to_snake(k)}" in self.__dir__() and v is not None
+                if (
+                    f"with_{camel_to_snake(k)}" in self.__dir__()
+                    or (k == "defined_tags" or "freeform_tags")
+                )
+                and v is not None
             }
             defaults.update(spec)
             super().__init__(defaults, **kwargs)
@@ -417,8 +440,7 @@ class DataFlow(Infrastructure):
         self.runtime = None
         self._name = None
 
-    @staticmethod
-    def _load_default_properties() -> dict:
+    def _load_default_properties(self) -> Dict:
         """
         Load default properties from environment variables, notebook session, etc.
 
@@ -428,30 +450,33 @@ class DataFlow(Infrastructure):
             a dictionary of default properties
         """
         defaults = {}
-        if "NB_SESSION_COMPARTMENT_OCID" in os.environ:
-            defaults["compartment_id"] = os.environ["NB_SESSION_COMPARTMENT_OCID"]
-        if "NB_SESSION_OCID" in os.environ:
+        if NB_SESSION_COMPARTMENT_OCID:
+            defaults[self.CONST_COMPARTMENT_ID] = NB_SESSION_COMPARTMENT_OCID
+        if NB_SESSION_OCID:
             dsc_client = OCIClientFactory(**default_signer()).data_science
             try:
-                nb_session = dsc_client.get_notebook_session(
-                    os.environ["NB_SESSION_OCID"]
-                ).data
+                nb_session = dsc_client.get_notebook_session(NB_SESSION_OCID).data
                 nb_config = nb_session.notebook_session_configuration_details
-                defaults["driver_shape"] = nb_config.shape
+
+                defaults[self.CONST_DRIVER_SHAPE] = nb_config.shape
                 logger.debug(f"Set driver shape to {nb_config.shape}")
-                defaults["executor_shape"] = nb_config.shape
+
+                defaults[self.CONST_EXECUTOR_SHAPE] = nb_config.shape
                 logger.debug(f"Set executor shape to {nb_config.shape}")
+
                 if nb_config.notebook_session_shape_config_details:
                     notebook_shape_config_details = oci_util.to_dict(
                         nb_config.notebook_session_shape_config_details
                     )
-                    defaults["driver_shape_config"] = copy.deepcopy(
+
+                    defaults[self.CONST_DRIVER_SHAPE_CONFIG] = copy.deepcopy(
                         notebook_shape_config_details
                     )
                     logger.debug(
                         f"Set driver shape config to {nb_config.notebook_session_shape_config_details}"
                     )
-                    defaults["executor_shape_config"] = copy.deepcopy(
+
+                    defaults[self.CONST_EXECUTOR_SHAPE_CONFIG] = copy.deepcopy(
                         notebook_shape_config_details
                     )
                     logger.debug(
@@ -463,11 +488,13 @@ class DataFlow(Infrastructure):
                     f"Error fetching details about Notebook session: {os.environ['NB_SESSION_OCID']}. {e}"
                 )
 
-        defaults["language"] = "PYTHON"
-        defaults["spark_version"] = "3.2.1"
-        defaults["num_executors"] = 1
-        logger.debug("Set spark version to be 3.2.1.")
-        logger.debug("Set number of executors to be 1.")
+        defaults["language"] = DEFAULT_LANGUAGE
+        defaults["spark_version"] = DEFAULT_SPARK_VERSION
+        defaults["num_executors"] = DEFAULT_NUM_EXECUTORS
+
+        logger.debug(f"Set spark version to be {defaults['spark_version']}")
+        logger.debug(f"Set number of executors to be {defaults['num_executors']}")
+
         return defaults
 
     @property
@@ -766,8 +793,54 @@ class DataFlow(Infrastructure):
         """
         return self.set_spec(self.CONST_PRIVATE_ENDPOINT_ID, private_endpoint_id)
 
+    def with_freeform_tag(self, **kwargs) -> "DataFlow":
+        """Sets freeform tags
+
+        Returns
+        -------
+        DataFlow
+            The DataFlow instance (self)
+        """
+        return self.set_spec(self.CONST_FREEFORM_TAGS, kwargs)
+
+    def with_defined_tag(self, **kwargs) -> "DataFlow":
+        """Sets defined tags
+
+        Returns
+        -------
+        DataFlow
+            The DataFlow instance (self)
+        """
+        return self.set_spec(self.CONST_DEFINED_TAGS, kwargs)
+
+    def with_pool_id(self, pool_id: str) -> "DataFlow":
+        """
+        Set the Data Flow Pool Id for a Data Flow job.
+
+        Parameters
+        ----------
+        pool_id: str
+            The OCID of a Data Flow Pool.
+
+        Returns
+        -------
+        DataFlow
+            the Data Flow instance itself
+        """
+        if not hasattr(CreateApplicationDetails, "pool_id"):
+            raise EnvironmentError(
+                "Data Flow Pool has not been supported in the current OCI SDK installed."
+            )
+        return self.set_spec(self.CONST_POOL_ID, pool_id)
+
     def __getattr__(self, item):
-        if f"with_{item}" in self.__dir__():
+        if item == self.CONST_DEFINED_TAGS or item == self.CONST_FREEFORM_TAGS:
+            return self.get_spec(item)
+        elif (
+            f"with_{item}" in self.__dir__()
+            and item != "defined_tag"
+            and item != "freeform_tag"
+        ):
             return self.get_spec(item)
         raise AttributeError(f"Attribute {item} not found.")
 
@@ -787,6 +860,11 @@ class DataFlow(Infrastructure):
         DataFlow
             a Data Flow job instance
         """
+        if self.pool_id:
+            if not hasattr(CreateApplicationDetails, "pool_id"):
+                raise EnvironmentError(
+                    "Data Flow Pool has not been supported in the current OCI SDK installed."
+                )
         # Set default display_name if not specified - randomly generated easy to remember name
         if not self.name:
             self.name = utils.get_random_name_for_resource()
@@ -839,7 +917,8 @@ class DataFlow(Infrastructure):
             {
                 "display_name": self.name,
                 "file_uri": runtime.script_uri,
-                "freeform_tags": runtime.freeform_tags,
+                "freeform_tags": runtime.freeform_tags or self.freeform_tags,
+                "defined_tags": runtime.defined_tags or self.defined_tags,
                 "archive_uri": runtime.archive_uri,
                 "configuration": runtime.configuration,
             }
@@ -850,12 +929,39 @@ class DataFlow(Infrastructure):
             raise ValueError(
                 "Compartment id is required. Specify compartment id via 'with_compartment_id()'."
             )
+        self._validate_shapes(payload)
         payload.pop("id", None)
         logger.debug(f"Creating a DataFlow Application with payload {payload}")
         self.df_app = DataFlowApp(**payload).create()
         self.with_id(self.df_app.id)
         return self
 
+    @staticmethod
+    def _validate_shapes(payload: Dict):
+        if "executor_shape" not in payload:
+            payload["executor_shape"] = DEFAULT_SHAPE
+        if "driver_shape" not in payload:
+            payload["driver_shape"] = DEFAULT_SHAPE
+        executor_shape = payload["executor_shape"]
+        executor_shape_config = payload.get("executor_shape_config", {})
+        driver_shape = payload["driver_shape"]
+        driver_shape_config = payload.get("driver_shape_config", {})
+        same_shape_family = False
+        for shape in DATAFLOW_SHAPE_FAMILY:
+            if shape in executor_shape and shape in driver_shape:
+                same_shape_family = True
+                break
+        if not same_shape_family:
+            raise ValueError(
+                "`executor_shape` and `driver_shape` must be from the same shape family."
+            )
+        if (not executor_shape.endswith("Flex") and executor_shape_config) or (
+            not driver_shape.endswith("Flex") and driver_shape_config
+        ):
+            raise ValueError(
+                "Shape config is not required for non flex shape from user end."
+            )
+
     @staticmethod
     def _upload_file(local_path, bucket, overwrite=False):
         signer = default_signer()
@@ -905,6 +1011,7 @@ class DataFlow(Infrastructure):
         args: List[str] = None,
         env_vars: Dict[str, str] = None,
         freeform_tags: Dict[str, str] = None,
+        defined_tags: Dict[str, Dict[str, object]] = None,
         wait: bool = False,
         **kwargs,
     ) -> DataFlowRun:
@@ -922,6 +1029,8 @@ class DataFlow(Infrastructure):
             dictionary of environment variables (not used for data flow)
         freeform_tags: Dict[str, str], optional
             freeform tags
+        defined_tags: Dict[str, Dict[str, object]], optional
+            defined tags
         wait: bool, optional
             whether to wait for a run to terminate
         kwargs
@@ -940,7 +1049,8 @@ class DataFlow(Infrastructure):
         # Set default display_name if not specified - randomly generated easy to remember name generated
         payload["display_name"] = name if name else utils.get_random_name_for_resource()
         payload["arguments"] = args if args and len(args) > 0 else None
-        payload["freeform_tags"] = freeform_tags
+        payload["freeform_tags"] = freeform_tags or self.freeform_tags
+        payload["defined_tags"] = defined_tags or self.defined_tags
         payload.pop("spark_version", None)
         logger.debug(f"Creating a DataFlow Run with payload {payload}")
         run = DataFlowRun(**payload).create()
@@ -1034,7 +1144,7 @@ class DataFlow(Infrastructure):
             for job in DataFlowApp.list_resource(compartment_id, **kwargs)
         ]
 
-    def to_dict(self) -> dict:
+    def to_dict(self, **kwargs) -> dict:
         """
         Serialize job to a dictionary.
 
@@ -1113,7 +1223,7 @@ class DataFlow(Infrastructure):
                 spec[shape_config] = copy.deepcopy(temp_maps)
         return spec
 
-    def to_yaml(self) -> str:
+    def to_yaml(self, **kwargs) -> str:
         """Serializes the object into YAML string.
 
         Returns
@@ -1121,4 +1231,22 @@ class DataFlow(Infrastructure):
         str
             YAML stored in a string.
         """
-        return yaml.safe_dump(self.to_dict())
+        return yaml.safe_dump(self.to_dict(**kwargs))
+
+    def init(self, **kwargs) -> "DataFlow":
+        """Initializes a starter specification for the DataFlow.
+
+        Returns
+        -------
+        DataFlow
+            The DataFlow instance (self)
+        """
+        return (
+            self.build()
+            .with_compartment_id(self.compartment_id or "{Provide a compartment OCID}")
+            .with_language(self.language or DEFAULT_LANGUAGE)
+            .with_spark_version(self.spark_version or DEFAULT_SPARK_VERSION)
+            .with_num_executors(self.num_executors or DEFAULT_NUM_EXECUTORS)
+            .with_driver_shape(self.driver_shape or DEFAULT_SHAPE)
+            .with_executor_shape(self.executor_shape or DEFAULT_SHAPE)
+        )
