@@ -5,12 +5,11 @@
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 from __future__ import annotations
 
-import json
 import os
-from typing import Any, Dict
+from typing import Dict
 
-from ads.jobs.builders.runtimes.base import Runtime
 from ads.common.auth import default_signer
+from ads.jobs.builders.runtimes.base import Runtime
 from ads.opctl.config.utils import convert_notebook
 
 
@@ -117,6 +116,33 @@ class CondaRuntime(Runtime):
             conda_spec[self.CONST_CONDA_REGION] = region
         return self.set_spec(self.CONST_CONDA, conda_spec)
 
+    def init(self, **kwargs) -> "CondaRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Parameters
+        ----------
+        **kwargs: Dict
+            - conda_slug: str
+                The conda environment slug.
+                If it contains '/', then the assumption that this is a custom conda environment.
+
+        Returns
+        -------
+        CondaRuntime
+            The runtime instance.
+        """
+        super().init(**kwargs)
+
+        conda_slug = kwargs.get("conda_slug", "")
+
+        if "/" not in conda_slug:
+            return self.with_service_conda(conda_slug)
+
+        return self.with_custom_conda(
+            conda_slug
+            or "{Path to the custom conda environment. Example: oci://bucket@namespace/prefix}"
+        )
+
 
 class ScriptRuntime(CondaRuntime):
     """Represents job runtime with scripts and conda pack.
@@ -136,7 +162,7 @@ class ScriptRuntime(CondaRuntime):
             # Environment variable
             .with_environment_variable(NAME="Welcome to OCI Data Science.")
             # Command line argument
-            .with_argument("100 linux \"hi there\"")
+            .with_argument("100 linux 'hi there'")
             # The entrypoint is applicable only to directory or zip file as source
             # The entrypoint should be a path relative to the working dir.
             # Here my_script.sh is a file in the code_dir/my_package directory
@@ -233,6 +259,23 @@ class ScriptRuntime(CondaRuntime):
         """
         return self.set_spec(self.CONST_ENTRYPOINT, entrypoint)
 
+    def init(self, **kwargs) -> "ScriptRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        ScriptRuntime
+            The runtime instance.
+        """
+        super().init(**kwargs)
+        return (
+            self.with_entrypoint("{For MLflow and Operator will be auto generated}")
+            .with_script(
+                "{Path to the script. For MLflow and Operator will be auto generated}"
+            )
+            .with_argument(**kwargs.get("args", {}))
+        )
+
 
 class _PythonRuntimeMixin(Runtime):
     CONST_OUTPUT_DIR = "outputDir"
@@ -403,6 +446,23 @@ class PythonRuntime(ScriptRuntime, _PythonRuntimeMixin):
     attribute_map.update(ScriptRuntime.attribute_map)
     attribute_map.update(_PythonRuntimeMixin.attribute_map)
 
+    def init(self, **kwargs) -> "PythonRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        PythonRuntime
+            The runtime instance.
+        """
+        super().init(**kwargs)
+        return (
+            self.with_working_dir("{For MLflow and Operator will be auto generated}")
+            .with_entrypoint("{For MLflow and Operator will be auto generated}")
+            .with_script(
+                "{Path to the script. For MLflow and Operator will be auto generated}"
+            )
+        )
+
 
 class NotebookRuntime(CondaRuntime):
     """Represents a job runtime with Jupyter notebook
@@ -572,6 +632,20 @@ class NotebookRuntime(CondaRuntime):
         """The path of the notebook relative to the source."""
         return self.get_spec(self.CONST_ENTRYPOINT)
 
+    def init(self, **kwargs) -> "NotebookRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        NotebookRuntime
+            The runtime instance.
+        """
+        super().init(**kwargs)
+        return self.with_source(
+            uri="{Path to the source code directory. For MLflow, it will be replaced with the path to the project}",
+            notebook="{Entrypoint notebook. For MLflow, it will be replaced with the CMD}",
+        ).with_exclude_tag("tag1")
+
 
 class GitPythonRuntime(CondaRuntime, _PythonRuntimeMixin):
     """Represents a job runtime with source code from git repository
@@ -679,9 +753,21 @@ class GitPythonRuntime(CondaRuntime, _PythonRuntimeMixin):
         """The OCID of the OCI Vault secret storing the Git SSH key."""
         return self.get_spec(self.CONST_GIT_SSH_SECRET_ID)
 
+    def init(self, **kwargs) -> "GitPythonRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        GitPythonRuntime
+            The runtime instance.
+        """
+        super().init(**kwargs)
+        return self.with_source(
+            "{Git URI. For MLflow and Operator will be auto generated}"
+        ).with_entrypoint("{For MLflow and Operator will be auto generated}")
+
 
 class DataFlowRuntime(CondaRuntime):
-
     CONST_SCRIPT_BUCKET = "scriptBucket"
     CONST_ARCHIVE_BUCKET = "archiveBucket"
     CONST_ARCHIVE_URI = "archiveUri"
@@ -888,6 +974,31 @@ class DataFlowRuntime(CondaRuntime):
     def convert(self, **kwargs):
         pass
 
+    def init(self, **kwargs) -> "DataFlowRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        DataFlowRuntime
+            The runtime instance.
+        """
+        super().init(**kwargs)
+        self._spec.pop(self.CONST_ENV_VAR, None)
+        return (
+            self.with_script_uri(
+                "{Path to the executable script. For MLflow and Operator will auto generated}"
+            )
+            .with_script_bucket(
+                kwargs.get(
+                    "script_bucket",
+                    "{The object storage bucket to save a script. "
+                    "Example: oci://<bucket_name>@<tenancy>/<prefix>}",
+                )
+            )
+            .with_overwrite(True)
+            .with_configuration({"spark.driverEnv.env_key": "env_value"})
+        )
+
 
 class DataFlowNotebookRuntime(DataFlowRuntime, NotebookRuntime):
     def convert(self, overwrite=False):
