@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*--
 
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 from ads.model.generic_model import GenericModel, ModelState
@@ -14,12 +14,41 @@ from ads.model.framework.sklearn_model import SklearnModel
 from ads.model.framework.tensorflow_model import TensorFlowModel
 from ads.model.framework.xgboost_model import XGBoostModel
 from ads.model.framework.spark_model import SparkPipelineModel
+from ads.model.framework.huggingface_model import HuggingFacePipelineModel
 
 from ads.model.deployment.model_deployer import ModelDeployer
 from ads.model.deployment.model_deployment import ModelDeployment
 from ads.model.deployment.model_deployment_properties import ModelDeploymentProperties
+
+from ads.model.serde.common import SERDE
+from ads.model.serde.model_input import ModelInputSerializer
+
 from ads.model.model_version_set import ModelVersionSet, experiment
 from ads.model.service.oci_datascience_model_version_set import (
     ModelVersionSetNotExists,
     ModelVersionSetNotSaved,
 )
+
+__all__ = [
+    "GenericModel",
+    "ModelState",
+    "DataScienceModel",
+    "ModelProperties",
+    "AutoMLModel",
+    "LightGBMModel",
+    "PyTorchModel",
+    "SklearnModel",
+    "TensorFlowModel",
+    "XGBoostModel",
+    "SparkPipelineModel",
+    "HuggingFacePipelineModel",
+    "ModelDeployer",
+    "ModelDeployment",
+    "ModelDeploymentProperties",
+    "SERDE",
+    "ModelInputSerializer",
+    "ModelVersionSet",
+    "experiment",
+    "ModelVersionSetNotExists",
+    "ModelVersionSetNotSaved",
+]
