@@ -119,19 +119,14 @@ def send_request(
     Returns:
         A JSON representive of a requests.Response object.
     """
-    headers = dict()
     if is_json_payload:
-        headers["Content-Type"] = (
-            header.get("content_type") or DEFAULT_CONTENT_TYPE_JSON
-        )
+        header["Content-Type"] =  header.pop("content_type", DEFAULT_CONTENT_TYPE_JSON) or DEFAULT_CONTENT_TYPE_JSON
         request_kwargs = {"json": data}
     else:
-        headers["Content-Type"] = (
-            header.get("content_type") or DEFAULT_CONTENT_TYPE_BYTES
-        )
+        header["Content-Type"] = header.pop("content_type", DEFAULT_CONTENT_TYPE_BYTES) or DEFAULT_CONTENT_TYPE_BYTES
         request_kwargs = {"data": data}  # should pass bytes when using data
-
-    request_kwargs["headers"] = headers
+    
+    request_kwargs["headers"] = header
 
     if dry_run:
         request_kwargs["headers"]["Accept"] = "*/*"
@@ -140,7 +135,7 @@ def send_request(
             return json.loads(req.body)
         return req.body
     else:
-        request_kwargs["auth"] = header.get("signer")
+        request_kwargs["auth"] = header.pop("signer")
         return requests.post(endpoint, **request_kwargs).json()
 
 
