@@ -5,12 +5,28 @@
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/from typing import Dict
 
 
+import copy
+import logging
+import traceback
 from typing import Any, Dict
+
+import oci.util as oci_util
+
+from ads.common.oci_datascience import DSCNotebookSession
+from ads.config import COMPARTMENT_OCID, NB_SESSION_OCID, PROJECT_OCID
 from ads.jobs.builders.base import Builder
 
 MODEL_DEPLOYMENT_INFRASTRUCTURE_TYPE = "datascienceModelDeployment"
 MODEL_DEPLOYMENT_INFRASTRUCTURE_KIND = "infrastructure"
 
+DEFAULT_BANDWIDTH_MBPS = 10
+DEFAULT_REPLICA = 1
+DEFAULT_SHAPE_NAME = "VM.Standard.E4.Flex"
+DEFAULT_OCPUS = 1
+DEFAULT_MEMORY_IN_GBS = 16
+
+logger = logging.getLogger(__name__)
+
 
 class ModelDeploymentInfrastructure(Builder):
     """A class used to represent a Model Deployment Infrastructure.
@@ -39,6 +55,8 @@ class ModelDeploymentInfrastructure(Builder):
         The access and predict log id of model deployment
     web_concurrency: int
         The web concurrency of model deployment
+    subnet_id: str
+        The subnet id of model deployment
 
     Methods
     -------
@@ -64,6 +82,8 @@ class ModelDeploymentInfrastructure(Builder):
         Sets the access and predict log id of model deployment
     with_web_concurrency(web_concurrency)
         Sets the web concurrency of model deployment
+    with_subnet_id(subnet_id)
+        Sets the subnet id of model deployment
 
     Example
     -------
@@ -79,6 +99,7 @@ class ModelDeploymentInfrastructure(Builder):
     ...        .with_replica(1)
     ...        .with_bandwidth_mbps(10)
     ...        .with_web_concurrency(10)
+    ...        .with_subnet_id(<subnet_id>)
     ...        .with_access_log(
     ...            log_group_id=<log_group_id>,
     ...            log_id=<log_id>
@@ -121,6 +142,7 @@ class ModelDeploymentInfrastructure(Builder):
     CONST_LOG_GROUP_ID = "logGroupId"
     CONST_WEB_CONCURRENCY = "webConcurrency"
     CONST_STREAM_CONFIG_DETAILS = "streamConfigurationDetails"
+    CONST_SUBNET_ID = "subnetId"
 
     attribute_map = {
         CONST_PROJECT_ID: "project_id",
@@ -136,6 +158,7 @@ class ModelDeploymentInfrastructure(Builder):
         CONST_LOG_ID: "log_id",
         CONST_LOG_GROUP_ID: "log_group_id",
         CONST_WEB_CONCURRENCY: "web_concurrency",
+        CONST_SUBNET_ID: "subnet_id",
     }
 
     shape_config_details_attribute_map = {
@@ -162,6 +185,7 @@ class ModelDeploymentInfrastructure(Builder):
         CONST_COMPARTMENT_ID: "compartment_id",
         CONST_SHAPE_NAME: f"{MODEL_CONFIG_DETAILS_PATH}.instance_configuration.instance_shape_name",
         CONST_SHAPE_CONFIG_DETAILS: f"{MODEL_CONFIG_DETAILS_PATH}.instance_configuration.model_deployment_instance_shape_config_details",
+        CONST_SUBNET_ID: f"{MODEL_CONFIG_DETAILS_PATH}.instance_configuration.subnet_id",
         CONST_REPLICA: f"{MODEL_CONFIG_DETAILS_PATH}.scaling_policy.instance_count",
         CONST_BANDWIDTH_MBPS: f"{MODEL_CONFIG_DETAILS_PATH}.bandwidth_mbps",
         CONST_ACCESS_LOG: "category_log_details.access",
@@ -179,6 +203,50 @@ class ModelDeploymentInfrastructure(Builder):
     def __init__(self, spec: Dict = None, **kwargs) -> None:
         super().__init__(spec, **kwargs)
 
+    def _load_default_properties(self) -> Dict:
+        """Load default properties from environment variables, notebook session, etc.
+
+        Returns
+        -------
+        Dict
+            A dictionary of default properties.
+        """
+        defaults = super()._load_default_properties()
+        if COMPARTMENT_OCID:
+            defaults[self.CONST_COMPARTMENT_ID] = COMPARTMENT_OCID
+        if PROJECT_OCID:
+            defaults[self.CONST_PROJECT_ID] = PROJECT_OCID
+
+        defaults[self.CONST_BANDWIDTH_MBPS] = DEFAULT_BANDWIDTH_MBPS
+        defaults[self.CONST_REPLICA] = DEFAULT_REPLICA
+
+        if NB_SESSION_OCID:
+            nb_session = None
+            try:
+                nb_session = DSCNotebookSession.from_ocid(NB_SESSION_OCID)
+            except Exception as e:
+                logger.warning(
+                    f"Error fetching details about Notebook "
+                    f"session: {NB_SESSION_OCID}. {e}"
+                )
+                logger.debug(traceback.format_exc())
+
+            nb_config = getattr(
+                nb_session, "notebook_session_config_details", None
+            ) or getattr(nb_session, "notebook_session_configuration_details", None)
+            if nb_config:
+                defaults[self.CONST_SHAPE_NAME] = nb_config.shape
+
+                if nb_config.notebook_session_shape_config_details:
+                    notebook_shape_config_details = oci_util.to_dict(
+                        nb_config.notebook_session_shape_config_details
+                    )
+                    defaults[self.CONST_SHAPE_CONFIG_DETAILS] = copy.deepcopy(
+                        notebook_shape_config_details
+                    )
+
+        return defaults
+
     @property
     def kind(self) -> str:
         """The kind of the object as showing in YAML.
@@ -518,3 +586,52 @@ class ModelDeploymentInfrastructure(Builder):
             The ModelDeploymentInfrastructure instance (self).
         """
         return self.set_spec(self.CONST_WEB_CONCURRENCY, web_concurrency)
+
+    def with_subnet_id(self, subnet_id: str) -> "ModelDeploymentInfrastructure":
+        """Sets the subnet id of model deployment.
+
+        Parameters
+        ----------
+        subnet_id : str
+            The subnet id of model deployment.
+
+        Returns
+        -------
+        ModelDeploymentInfrastructure
+            The ModelDeploymentInfrastructure instance (self).
+        """
+        return self.set_spec(self.CONST_SUBNET_ID, subnet_id)
+
+    @property
+    def subnet_id(self) -> str:
+        """The model deployment subnet id.
+
+        Returns
+        -------
+        str
+            The model deployment subnet id.
+        """
+        return self.get_spec(self.CONST_SUBNET_ID, None)
+
+    def init(self, **kwargs) -> "ModelDeploymentInfrastructure":
+        """Initializes a starter specification for the ModelDeploymentInfrastructure.
+
+        Returns
+        -------
+        ModelDeploymentInfrastructure
+            The ModelDeploymentInfrastructure instance (self)
+        """
+        return (
+            self.build()
+            .with_compartment_id(self.compartment_id or "{Provide a compartment OCID}")
+            .with_project_id(self.project_id or "{Provide a project OCID}")
+            .with_bandwidth_mbps(self.bandwidth_mbps or DEFAULT_BANDWIDTH_MBPS)
+            .with_replica(self.replica or DEFAULT_REPLICA)
+            .with_shape_name(self.shape_name or DEFAULT_SHAPE_NAME)
+            .with_shape_config_details(
+                ocpus=self.shape_config_details.get(self.CONST_OCPUS, DEFAULT_OCPUS),
+                memory_in_gbs=self.shape_config_details.get(
+                    self.CONST_MEMORY_IN_GBS, DEFAULT_MEMORY_IN_GBS
+                ),
+            )
+        )
