@@ -21,6 +21,11 @@ class OCIModelDeploymentRuntimeType:
     CONTAINER = "OCIR_CONTAINER"
 
 
+class ModelDeploymentMode:
+    HTTPS = "HTTPS_ONLY"
+    STREAM = "STREAM_ONLY"
+
+
 class ModelDeploymentRuntime(Builder):
     """A class used to represent a Model Deployment Runtime.
 
@@ -36,6 +41,18 @@ class ModelDeploymentRuntime(Builder):
         The output stream ids of model deployment.
     model_uri: str
         The model uri of model deployment.
+    bucket_uri: str
+        The OCI Object Storage URI where large size model artifacts will be copied to.
+    auth: Dict
+        The default authentication is set using `ads.set_auth` API.
+    region: str
+        The destination Object Storage bucket region.
+    overwrite_existing_artifact: bool
+        Whether overwrite existing target bucket artifact or not.
+    remove_existing_artifact: bool
+        Whether artifacts uploaded to object storage bucket need to be removed or not.
+    timeout: int
+        The connection timeout in seconds for the client.
 
     Methods
     -------
@@ -49,6 +66,18 @@ class ModelDeploymentRuntime(Builder):
         Sets the output stream ids of model deployment
     with_model_uri(model_uri)
         Sets the model uri of model deployment
+    with_bucket_uri(bucket_uri)
+        Sets the bucket uri when uploading large size model.
+    with_auth(auth)
+        Sets the default authentication when uploading large size model.
+    with_region(region)
+        Sets the region when uploading large size model.
+    with_overwrite_existing_artifact(overwrite_existing_artifact)
+        Sets whether to overwrite existing artifact when uploading large size model.
+    with_remove_existing_artifact(remove_existing_artifact)
+        Sets whether to remove existing artifact when uploading large size model.
+    with_timeout(timeout)
+        Sets the connection timeout when uploading large size model.
     """
 
     CONST_MODEL_ID = "modelId"
@@ -60,6 +89,12 @@ class ModelDeploymentRuntime(Builder):
     CONST_INPUT_STREAM_IDS = "inputStreamIds"
     CONST_OUTPUT_STREAM_IDS = "outputStreamIds"
     CONST_ENVIRONMENT_CONFIG_DETAILS = "environmentConfigurationDetails"
+    CONST_BUCKET_URI = "bucketUri"
+    CONST_AUTH = "auth"
+    CONST_REGION = "region"
+    CONST_OVERWRITE_EXISTING_ARTIFACT = "overwriteExistingArtifact"
+    CONST_REMOVE_EXISTING_ARTIFACT = "removeExistingArtifact"
+    CONST_TIMEOUT = "timeout"
 
     attribute_map = {
         CONST_ENV: "env",
@@ -68,6 +103,12 @@ class ModelDeploymentRuntime(Builder):
         CONST_OUTPUT_STREAM_IDS: "output_stream_ids",
         CONST_DEPLOYMENT_MODE: "deployment_mode",
         CONST_MODEL_URI: "model_uri",
+        CONST_BUCKET_URI: "bucket_uri",
+        CONST_AUTH: "auth",
+        CONST_REGION: "region",
+        CONST_OVERWRITE_EXISTING_ARTIFACT: "overwrite_existing_artifact",
+        CONST_REMOVE_EXISTING_ARTIFACT: "remove_existing_artifact",
+        CONST_TIMEOUT: "timeout",
     }
 
     ENVIRONMENT_CONFIG_DETAILS_PATH = (
@@ -137,7 +178,7 @@ class ModelDeploymentRuntime(Builder):
         str
             The deployment mode of model deployment.
         """
-        return self.get_spec(self.CONST_DEPLOYMENT_MODE, None)
+        return self.get_spec(self.CONST_DEPLOYMENT_MODE, ModelDeploymentMode.HTTPS)
 
     def with_deployment_mode(self, deployment_mode: str) -> "ModelDeploymentRuntime":
         """Sets the deployment mode of model deployment.
@@ -237,6 +278,184 @@ class ModelDeploymentRuntime(Builder):
         """
         return self.set_spec(self.CONST_MODEL_URI, model_uri)
 
+    @property
+    def bucket_uri(self) -> str:
+        """The bucket uri of model.
+
+        Returns
+        -------
+        str
+            The bucket uri of model.
+        """
+        return self.get_spec(self.CONST_BUCKET_URI, None)
+
+    def with_bucket_uri(self, bucket_uri: str) -> "ModelDeploymentRuntime":
+        """Sets the bucket uri of model.
+
+        Parameters
+        ----------
+        bucket_uri: str
+            The bucket uri of model.
+
+        Returns
+        -------
+        ModelDeploymentRuntime
+            The ModelDeploymentRuntime instance (self).
+        """
+        return self.set_spec(self.CONST_BUCKET_URI, bucket_uri)
+
+    @property
+    def auth(self) -> Dict:
+        """The auth when uploading large-size model.
+
+        Returns
+        -------
+        Dict
+            The auth when uploading large-size model.
+        """
+        return self.get_spec(self.CONST_AUTH, {})
+
+    def with_auth(self, auth: Dict) -> "ModelDeploymentRuntime":
+        """Sets the auth when uploading large-size model.
+
+        Parameters
+        ----------
+        auth: Dict
+            The auth when uploading large-size model.
+
+        Returns
+        -------
+        ModelDeploymentRuntime
+            The ModelDeploymentRuntime instance (self).
+        """
+        return self.set_spec(self.CONST_AUTH, auth)
+
+    @property
+    def region(self) -> str:
+        """The region when uploading large-size model.
+
+        Returns
+        -------
+        str
+            The region when uploading large-size model.
+        """
+        return self.get_spec(self.CONST_REGION, None)
+
+    def with_region(self, region: str) -> "ModelDeploymentRuntime":
+        """Sets the region when uploading large-size model.
+
+        Parameters
+        ----------
+        region: str
+            The region when uploading large-size model.
+
+        Returns
+        -------
+        ModelDeploymentRuntime
+            The ModelDeploymentRuntime instance (self).
+        """
+        return self.set_spec(self.CONST_REGION, region)
+
+    @property
+    def overwrite_existing_artifact(self) -> bool:
+        """Overwrite existing artifact when uploading large size model.
+
+        Returns
+        -------
+        bool
+            Overwrite existing artifact when uploading large size model.
+        """
+        return self.get_spec(self.CONST_OVERWRITE_EXISTING_ARTIFACT, True)
+
+    def with_overwrite_existing_artifact(
+        self, overwrite_existing_artifact: bool
+    ) -> "ModelDeploymentRuntime":
+        """Sets whether to overwrite existing artifact when uploading large size model.
+
+        Parameters
+        ----------
+        overwrite_existing_artifact: bool
+            Overwrite existing artifact when uploading large size model.
+
+        Returns
+        -------
+        ModelDeploymentRuntime
+            The ModelDeploymentRuntime instance (self).
+        """
+        return self.set_spec(
+            self.CONST_OVERWRITE_EXISTING_ARTIFACT, overwrite_existing_artifact
+        )
+
+    @property
+    def remove_existing_artifact(self) -> bool:
+        """Remove existing artifact when uploading large size model.
+
+        Returns
+        -------
+        bool
+            Remove existing artifact when uploading large size model.
+        """
+        return self.get_spec(self.CONST_REMOVE_EXISTING_ARTIFACT, True)
+
+    def with_remove_existing_artifact(
+        self, remove_existing_artifact: bool
+    ) -> "ModelDeploymentRuntime":
+        """Sets whether to remove existing artifact when uploading large size model.
+
+        Parameters
+        ----------
+        remove_existing_artifact: bool
+            Remove existing artifact when uploading large size model.
+
+        Returns
+        -------
+        ModelDeploymentRuntime
+            The ModelDeploymentRuntime instance (self).
+        """
+        return self.set_spec(
+            self.CONST_REMOVE_EXISTING_ARTIFACT, remove_existing_artifact
+        )
+
+    @property
+    def timeout(self) -> int:
+        """The timeout when uploading large-size model.
+
+        Returns
+        -------
+        int
+            The timeout when uploading large-size model.
+        """
+        return self.get_spec(self.CONST_TIMEOUT, None)
+
+    def with_timeout(self, timeout: int) -> "ModelDeploymentRuntime":
+        """Sets the timeout when uploading large-size model.
+
+        Parameters
+        ----------
+        timeout: int
+            The timeout when uploading large-size model.
+
+        Returns
+        -------
+        ModelDeploymentRuntime
+            The ModelDeploymentRuntime instance (self).
+        """
+        return self.set_spec(self.CONST_TIMEOUT, timeout)
+
+    def init(self, **kwargs) -> "ModelDeploymentRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        Self
+            This method returns self to support chaining methods.
+        """
+        return (
+            self.with_env({"key": "value"})
+            .with_deployment_mode("HTTPS_ONLY")
+            .with_model_uri("<MODEL_URI>")
+        )
+
 
 class ModelDeploymentCondaRuntime(ModelDeploymentRuntime):
     """A class used to represent a Model Deployment Conda Runtime.
@@ -272,6 +491,16 @@ class ModelDeploymentCondaRuntime(ModelDeploymentRuntime):
         """
         return OCIModelDeploymentRuntimeType.CONDA
 
+    def init(self, **kwargs) -> "ModelDeploymentCondaRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        CondaRuntime
+            The runtime instance.
+        """
+        return super().init(**kwargs)
+
 
 class ModelDeploymentContainerRuntime(ModelDeploymentRuntime):
     """A class used to represent a Model Deployment Container Runtime.
@@ -330,6 +559,7 @@ class ModelDeploymentContainerRuntime(ModelDeploymentRuntime):
     CONST_ENTRYPOINT = "entrypoint"
     CONST_SERVER_PORT = "serverPort"
     CONST_HEALTH_CHECK_PORT = "healthCheckPort"
+    CONST_INFERENCE_SERVER = "inferenceServer"
 
     attribute_map = {
         **ModelDeploymentRuntime.attribute_map,
@@ -339,6 +569,7 @@ class ModelDeploymentContainerRuntime(ModelDeploymentRuntime):
         CONST_ENTRYPOINT: "entrypoint",
         CONST_SERVER_PORT: "server_port",
         CONST_HEALTH_CHECK_PORT: "health_check_port",
+        CONST_INFERENCE_SERVER: "inference_server",
     }
 
     payload_attribute_map = {
@@ -532,3 +763,76 @@ class ModelDeploymentContainerRuntime(ModelDeploymentRuntime):
             The ModelDeploymentContainerRuntime instance (self).
         """
         return self.set_spec(self.CONST_HEALTH_CHECK_PORT, health_check_port)
+
+    @property
+    def inference_server(self) -> str:
+        """Returns the inference server.
+
+        Returns
+        -------
+        str
+            The inference server.
+        """
+        return self.get_spec(self.CONST_INFERENCE_SERVER, None)
+
+    def with_inference_server(
+        self, inference_server: str = "triton"
+    ) -> "ModelDeploymentRuntime":
+        """Sets the inference server. Current supported inference server is "triton".
+        Note if you are using byoc, you do not need to set the inference server.
+
+        Parameters
+        ----------
+        inference_server: str
+            Set the inference server.
+
+        Returns
+        -------
+        ModelDeploymentRuntime
+            The ModelDeploymentRuntime instance (self).
+
+        Example
+        -------
+        >>> from ads.model.deployment import ModelDeployment, ModelDeploymentContainerRuntime, ModelDeploymentInfrastructure
+        >>> import ads
+        >>> ads.set_auth("resource_principal")
+        >>> infrastructure = ModelDeploymentInfrastructure()\
+        ...                 .with_project_id(<project_id>)\
+        ...                 .with_compartment_id(<comparment_id>)\
+        ...                 .with_shape_name("VM.Standard.E4.Flex")\
+        ...                 .with_replica(2)\
+        ...                 .with_bandwidth_mbps(10)\
+        ...                 .with_access_log(log_group_id=<deployment_log_group_id>, log_id=<deployment_access_log_id>)\
+        ...                 .with_predict_log(log_group_id=<deployment_log_group_id>, log_id=<deployment_predict_log_id>)
+
+        >>> runtime = ModelDeploymentContainerRuntime()\
+        ...                 .with_image(<container_image>)\
+        ...                 .with_server_port(<server_port>)\
+        ...                 .with_health_check_port(<health_check_port>)\
+        ...                 .with_model_uri(<model_id>)\
+        ...                 .with_env({"key":"value", "key2":"value2"})\
+        ...                 .with_inference_server("triton")
+        >>> deployment = ModelDeployment()\
+        ...                 .with_display_name("Triton Example")\
+        ...                 .with_infrastructure(infrastructure)\
+        ...                 .with_runtime(runtime)
+        >>> deployment.deploy()
+        """
+        return self.set_spec(self.CONST_INFERENCE_SERVER, inference_server.lower())
+
+    def init(self, **kwargs) -> "ModelDeploymentContainerRuntime":
+        """Initializes a starter specification for the runtime.
+
+        Returns
+        -------
+        CondaRuntime
+            The runtime instance.
+        """
+        super().init(**kwargs)
+        return (
+            self.with_image("iad.ocir.io/<namespace>/<image>:<tag>")
+            .with_image_digest("<IMAGE_DIGEST>")
+            .with_entrypoint(["python", "/opt/ds/model/deployed_model/api.py"])
+            .with_server_port(5000)
+            .with_health_check_port(5000)
+        )
