@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*--
 
-# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2024 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 import json
@@ -23,6 +23,7 @@ from ads.common import logger
 from ads.common.error import ChangesNotCommitted
 from ads.common.extended_enum import ExtendedEnumMeta
 from ads.common.serializer import DataClassSerializable
+from ads.common.object_storage_details import ObjectStorageDetails
 from oci.util import to_dict
 
 try:
@@ -1654,6 +1655,9 @@ class ModelProvenanceMetadata(DataClassSerializable):
         None
             Nothing.
         """
+        if ObjectStorageDetails.is_oci_path(path):
+            return
+
         if self.repo is not None and not ignore:
             path_abs = os.path.abspath(path)
             if (
