@@ -1,63 +1,67 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2022, 2023 Oracle and/or its affiliates.
+# Copyright (c) 2022, 2024 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
-import os
 import configparser
+import os
+from typing import Dict, List, Union
+
 import click
+import fsspec
+import yaml
+from ads.opctl.backend.marketplace.local_marketplace import (
+    LocalMarketplaceOperatorBackend,
+)
 
 import ads
-
-from typing import Dict, List
-
-from ads.common.auth import AuthContext
-from ads.common.oci_datascience import DSCNotebookSession
+from ads.common.auth import AuthContext, AuthType
 from ads.common.extended_enum import ExtendedEnumMeta
+from ads.common.oci_datascience import DSCNotebookSession
+from ads.opctl.backend.ads_dataflow import DataFlowBackend
 from ads.opctl.backend.ads_ml_job import MLJobBackend, MLJobDistributedBackend
+from ads.opctl.backend.ads_ml_pipeline import PipelineBackend
 from ads.opctl.backend.ads_model_deployment import ModelDeploymentBackend
 from ads.opctl.backend.local import (
     LocalBackend,
     LocalBackendDistributed,
+    LocalModelDeploymentBackend,
+    LocalOperatorBackend,
     LocalPipelineBackend,
 )
-from ads.opctl.backend.ads_dataflow import DataFlowBackend
-from ads.opctl.backend.ads_ml_pipeline import PipelineBackend
 from ads.opctl.config.base import ConfigProcessor
 from ads.opctl.config.merger import ConfigMerger
 from ads.opctl.config.resolver import ConfigResolver
 from ads.opctl.config.utils import read_from_ini
 from ads.opctl.config.validator import ConfigValidator
 from ads.opctl.config.yaml_parsers import YamlSpecParser
-import fsspec
-from ads.common.auth import AuthType
-
-from ads.opctl.distributed.cmds import (
-    update_ini,
-    increment_tag_in_ini,
-    docker_build_cmd,
-    update_image,
-    verify_and_publish_image,
-    update_config_image,
-)
 from ads.opctl.constants import (
+    ADS_CONFIG_FILE_NAME,
+    ADS_DATAFLOW_CONFIG_FILE_NAME,
+    ADS_JOBS_CONFIG_FILE_NAME,
+    ADS_LOCAL_BACKEND_CONFIG_FILE_NAME,
+    ADS_ML_PIPELINE_CONFIG_FILE_NAME,
+    ADS_MODEL_DEPLOYMENT_CONFIG_FILE_NAME,
+    BACKEND_NAME,
+    DEFAULT_ADS_CONFIG_FOLDER,
+    DEFAULT_CONDA_PACK_FOLDER,
     DEFAULT_OCI_CONFIG_FILE,
     DEFAULT_PROFILE,
-    DEFAULT_CONDA_PACK_FOLDER,
-    DEFAULT_ADS_CONFIG_FOLDER,
-    ADS_CONFIG_FILE_NAME,
-    ADS_JOBS_CONFIG_FILE_NAME,
-    ADS_DATAFLOW_CONFIG_FILE_NAME,
-    ADS_ML_PIPELINE_CONFIG_FILE_NAME,
-    ADS_LOCAL_BACKEND_CONFIG_FILE_NAME,
-    BACKEND_NAME,
+    RESOURCE_TYPE,
 )
-from ads.opctl.utils import (
-    is_in_notebook_session,
-    get_service_pack_prefix,
+from ads.opctl.distributed.cmds import (
+    docker_build_cmd,
+    increment_tag_in_ini,
+    update_config_image,
+    update_image,
+    update_ini,
+    verify_and_publish_image,
 )
-import yaml
+from ads.opctl.operator.common.backend_factory import (
+    BackendFactory as OperatorBackendFactory,
+)
+from ads.opctl.utils import get_service_pack_prefix, is_in_notebook_session
 
 
 class DataScienceResource(str, metaclass=ExtendedEnumMeta):
@@ -65,6 +69,7 @@ class DataScienceResource(str, metaclass=ExtendedEnumMeta):
     DATAFLOW = "dataflowapplication"
     PIPELINE = "datasciencepipeline"
     MODEL_DEPLOYMENT = "datasciencemodeldeployment"
+    MODEL = "datasciencemodel"
 
 
 class DataScienceResourceRun(str, metaclass=ExtendedEnumMeta):
@@ -82,6 +87,7 @@ DATA_SCIENCE_RESOURCE_BACKEND_MAP = {
     DataScienceResource.PIPELINE: "pipeline",
     DataScienceResourceRun.PIPELINE_RUN: "pipeline",
     DataScienceResourceRun.MODEL_DEPLOYMENT: "deployment",
+    DataScienceResource.MODEL: "deployment",
 }
 
 DATA_SCIENCE_RESOURCE_RUN_BACKEND_MAP = {
@@ -98,11 +104,14 @@ class _BackendFactory:
         BACKEND_NAME.DATAFLOW.value: DataFlowBackend,
         BACKEND_NAME.PIPELINE.value: PipelineBackend,
         BACKEND_NAME.MODEL_DEPLOYMENT.value: ModelDeploymentBackend,
+        BACKEND_NAME.OPERATOR_LOCAL.value: LocalOperatorBackend,
+        BACKEND_NAME.MARKETPLACE.value: LocalMarketplaceOperatorBackend,
     }
 
     LOCAL_BACKENDS_MAP = {
         BACKEND_NAME.JOB.value: LocalBackend,
         BACKEND_NAME.PIPELINE.value: LocalPipelineBackend,
+        BACKEND_NAME.MODEL_DEPLOYMENT.value: LocalModelDeploymentBackend,
     }
 
     def __init__(self, config: Dict):
@@ -119,7 +128,7 @@ class _BackendFactory:
     @property
     def backend(self):
         if self._backend == BACKEND_NAME.LOCAL.value:
-            kind = self.config.get("kind")
+            kind = self.config.get("kind") or self.config["execution"].get("kind")
             if kind not in self.LOCAL_BACKENDS_MAP:
                 options = [backend for backend in self.LOCAL_BACKENDS_MAP.keys()]
                 # Special case local backend option not supported by this factory.
@@ -141,7 +150,7 @@ def _save_yaml(yaml_content, **kwargs):
     yaml_content : str
         YAML content as string.
     """
-    if kwargs["job_info"]:
+    if kwargs.get("job_info"):
         yaml_path = os.path.abspath(os.path.expanduser(kwargs["job_info"]))
         if os.path.isfile(yaml_path):
             overwrite = input(
@@ -170,7 +179,31 @@ def run(config: Dict, **kwargs) -> Dict:
     Dict
         dictionary of job id and run id in case of ML Job run, else empty if running locally
     """
-    p = ConfigProcessor(config).step(ConfigMerger, **kwargs)
+    if config:
+        p = ConfigProcessor(config).step(ConfigMerger, **kwargs)
+        try:
+            return OperatorBackendFactory.backend(
+                config=p,
+                backend=p.config["execution"].get("backend"),
+                **{
+                    key: value
+                    for key, value in kwargs.items()
+                    if key not in ("backend", "config")
+                },
+            ).run(**kwargs)
+        except RuntimeError:
+            pass
+
+        if (
+            p.config["kind"] != BACKEND_NAME.LOCAL.value
+            and p.config["kind"] != "distributed"
+        ):
+            p.config["execution"]["backend"] = p.config["kind"]
+            return _BackendFactory(p.config).backend.apply()
+    else:
+        # If no yaml is provided and config is empty, we assume there's cmdline args to define a job.
+        config = {"kind": "job"}
+        p = ConfigProcessor(config).step(ConfigMerger, **kwargs)
     if config.get("kind") == "distributed":  # TODO: add kind factory
         print(
             "......................... Initializing the process ..................................."
@@ -210,7 +243,7 @@ def run(config: Dict, **kwargs) -> Dict:
                 "backend operator for distributed training can either be local or job"
             )
         else:
-            if not kwargs["dry_run"]:
+            if not kwargs["dry_run"] and not kwargs["nobuild"]:
                 verify_and_publish_image(kwargs["nopush"], config)
                 print("running image: " + config["spec"]["cluster"]["spec"]["image"])
             cluster_def = YamlSpecParser.parse_content(config)
@@ -242,13 +275,6 @@ def run(config: Dict, **kwargs) -> Dict:
                 _save_yaml(yamlContent, **kwargs)
             return cluster_run_info
     else:
-        if (
-            "kind" in p.config
-            and p.config["execution"].get("backend", None) != BACKEND_NAME.LOCAL.value
-        ):
-            p.config["execution"]["backend"] = p.config["kind"]
-            return _BackendFactory(p.config).backend.apply()
-
         if "ocid" in p.config["execution"]:
             resource_to_backend = {
                 DataScienceResource.JOB: BACKEND_NAME.JOB,
@@ -285,7 +311,6 @@ def run_diagnostics(config: Dict, **kwargs) -> Dict:
     """
     p = ConfigProcessor(config).step(ConfigMerger, **kwargs)
     if config.get("kind") == "distributed":  # TODO: add kind factory
-
         config = update_config_image(config)
         cluster_def = YamlSpecParser.parse_content(config)
 
@@ -321,31 +346,6 @@ def _update_env_vars(config, env_vars: List):
     return config
 
 
-def init_operator(**kwargs) -> str:
-    """
-    Initialize the resources for an operator
-
-    Parameters
-    ----------
-    kwargs: dict
-        keyword argument, stores command line args
-    Returns
-    -------
-    folder_path: str
-        a path to the folder with all of the resources
-    """
-    # TODO: confirm that operator slug is in the set of valid operator slugs
-    assert kwargs["operator_slug"] == "dask_cluster"
-
-    if kwargs.get("folder_path"):
-        kwargs["operator_folder_path"] = kwargs.pop("folder_path")[0]
-    else:
-        kwargs["operator_folder_path"] = kwargs["operator_slug"]
-    p = ConfigProcessor().step(ConfigMerger, **kwargs)
-    print(f"config check: {p.config}")
-    return _BackendFactory(p.config).backend.init_operator()
-
-
 def delete(**kwargs) -> None:
     """
     Delete a MLJob/DataFlow run.
@@ -374,7 +374,7 @@ def delete(**kwargs) -> None:
     ):
         kwargs["id"] = kwargs.pop("ocid")
     else:
-        raise ValueError(f"{kwargs['ocid']} is valid or supported.")
+        raise ValueError(f"{kwargs['ocid']} is invalid or not supported.")
 
     p = ConfigProcessor().step(ConfigMerger, **kwargs)
     return _BackendFactory(p.config).backend.delete()
@@ -388,13 +388,22 @@ def cancel(**kwargs) -> None:
     ----------
     kwargs: dict
         keyword argument, stores command line args
+
     Returns
     -------
     None
     """
-    kwargs["run_id"] = kwargs.pop("ocid")
-    if not kwargs.get("backend"):
-        kwargs["backend"] = _get_backend_from_run_id(kwargs["run_id"])
+    kwargs["backend"] = _get_backend_from_ocid(kwargs["ocid"])
+    if (
+        DataScienceResourceRun.JOB_RUN in kwargs["ocid"]
+        or DataScienceResourceRun.DATAFLOW_RUN in kwargs["ocid"]
+        or DataScienceResourceRun.PIPELINE_RUN in kwargs["ocid"]
+    ):
+        kwargs["run_id"] = kwargs.pop("ocid")
+    elif DataScienceResource.JOB in kwargs["ocid"]:
+        kwargs["id"] = kwargs.pop("ocid")
+    else:
+        raise ValueError(f"{kwargs['ocid']} is invalid or not supported.")
     p = ConfigProcessor().step(ConfigMerger, **kwargs)
     return _BackendFactory(p.config).backend.cancel()
 
@@ -458,6 +467,27 @@ def deactivate(**kwargs) -> None:
     return _BackendFactory(p.config).backend.deactivate()
 
 
+def predict(**kwargs) -> None:
+    """
+    Make prediction using the model with the payload.
+
+    Parameters
+    ----------
+    kwargs: dict
+        keyword argument, stores command line args
+
+    Returns
+    -------
+    None
+    """
+    p = ConfigProcessor().step(ConfigMerger, **kwargs)
+    if "datasciencemodeldeployment" in p.config["execution"].get("ocid", ""):
+        return ModelDeploymentBackend(p.config).predict()
+    else:
+        # model ocid or artifact directory
+        return LocalModelDeploymentBackend(p.config).predict()
+
+
 def init_vscode(**kwargs) -> None:
     """
     Create a .devcontainer.json file for local development.
@@ -511,6 +541,12 @@ def configure() -> None:
     if "CONDA" not in config_parser:
         config_parser["CONDA"] = {}
 
+    oci_auth = click.prompt(
+        text="Default OCI authentication type:",
+        type=click.Choice(AuthType.values()),
+        default=None,
+    )
+
     oci_config_path = click.prompt(
         "OCI config path:",
         default=config_parser["OCI"].get("oci_config", DEFAULT_OCI_CONFIG_FILE),
@@ -526,6 +562,7 @@ def configure() -> None:
     config_parser["OCI"] = {
         "oci_config": oci_config_path,
         "oci_profile": oci_profile,
+        "auth": oci_auth,
     }
     conda_pack_path = click.prompt(
         "Conda pack install folder:",
@@ -553,9 +590,10 @@ def configure() -> None:
         config_parser.write(f)
     print(f"Configuration saved at {os.path.join(folder, ADS_CONFIG_FILE_NAME)}")
 
-    print("==== Setting configuration for OCI Jobs ====")
+    print("==== Setting configuration for Data Science Jobs ====")
     if click.confirm(
-        f"Do you want to set up or update OCI Jobs configuration?", default=True
+        f"Do you want to set up or update Data Science Jobs configuration?",
+        default=True,
     ):
         required_fields = [
             ("compartment_id", ""),
@@ -570,6 +608,8 @@ def configure() -> None:
             ("log_id", ""),
             ("docker_registry", ""),
             ("conda_pack_os_prefix", "in the format oci://<bucket>@<namespace>/<path>"),
+            ("memory_in_gbs", ""),
+            ("ocpus", ""),
         ]
         _set_service_configurations(
             ADS_JOBS_CONFIG_FILE_NAME,
@@ -580,9 +620,9 @@ def configure() -> None:
             is_in_notebook_session(),
         )
 
-    print("==== Setting configuration for OCI DataFlow ====")
+    print("==== Setting configuration for OCI Data Flow ====")
     if click.confirm(
-        f"Do you want to set up or update OCI DataFlow configuration?", default=True
+        f"Do you want to set up or update OCI Data Flow configuration?", default=True
     ):
         required_fields = [
             ("compartment_id", ""),
@@ -596,6 +636,10 @@ def configure() -> None:
             ("num_executors", ""),
             ("spark_version", ""),
             ("archive_bucket", "in the format oci://<bucket>@<namespace>/<path>"),
+            ("driver_shape_memory_in_gbs", ""),
+            ("driver_shape_ocpus", ""),
+            ("executor_shape_memory_in_gbs", ""),
+            ("executor_shape_ocpus", ""),
         ]
         _set_service_configurations(
             ADS_DATAFLOW_CONFIG_FILE_NAME,
@@ -628,6 +672,36 @@ def configure() -> None:
             is_in_notebook_session(),
         )
 
+    print("==== Setting configuration for Data Science Model Deployment ====")
+    if click.confirm(
+        f"Do you want to set up or update Data Science Model Deployment configuration?",
+        default=True,
+    ):
+        required_fields = [
+            ("compartment_id", ""),
+            ("project_id", ""),
+            ("shape_name", ""),
+        ]
+
+        optional_fields = [
+            ("log_group_id", ""),
+            ("log_id", ""),
+            ("bandwidth_mbps", ""),
+            ("replica", ""),
+            ("web_concurrency", ""),
+            ("memory_in_gbs", ""),
+            ("ocpus", ""),
+        ]
+
+        _set_service_configurations(
+            ADS_MODEL_DEPLOYMENT_CONFIG_FILE_NAME,
+            required_fields,
+            optional_fields,
+            folder,
+            oci_config_path,
+            is_in_notebook_session(),
+        )
+
     print("==== Setting configuration for local backend ====")
     if click.confirm(
         f"Do you want to set up or update local backend configuration?", default=True
@@ -737,3 +811,63 @@ def _get_backend_from_run_id(ocid: str) -> str:
         if value in ocid:
             return DATA_SCIENCE_RESOURCE_RUN_BACKEND_MAP[value]
     raise ValueError("Must provide a resource run OCID.")
+
+
+def init(
+    resource_type: str,
+    runtime_type: Union[str, None] = None,
+    output: Union[str, None] = None,
+    overwrite: bool = False,
+    ads_config: str = DEFAULT_ADS_CONFIG_FOLDER,
+    **kwargs,
+) -> Union[str, None]:
+    """
+    Generates a starter specification template YAML for the Data Science resource.
+
+    Parameters
+    ----------
+    resource_type: str
+        The resource type to generate the specification YAML.
+    runtime_type: (str, optional). Defaults to None.
+        The resource runtime type.
+    output: (str, optional). Defaults to None.
+        The path to the file to save the resulting specification template YAML.
+    overwrite: (bool, optional). Defaults to False.
+        Whether to overwrite the result specification YAML if exists.
+    ads_config: (str, optional)
+        The folder where the ads opctl config located.
+    kwargs: (Dict, optional).
+        Any optional kwargs arguments.
+
+    Returns
+    -------
+    Union[str, None]
+        The YAML specification for the given resource if `output` was not provided,
+        path to the specification otherwise.
+
+    Raises
+    ------
+    ValueError
+        If `resource_type` not specified.
+    """
+    if not resource_type:
+        raise ValueError(
+            f"The `resource_type` must be specified. Supported values: {RESOURCE_TYPE.values()}"
+        )
+
+    # get config info form INI files
+    p = ConfigProcessor({"execution": {"backend": resource_type}}).step(
+        ConfigMerger, ads_config=ads_config or DEFAULT_ADS_CONFIG_FOLDER, **kwargs
+    )
+
+    # generate YAML specification template
+    spec_yaml = _BackendFactory(p.config).backend.init(
+        uri=output, overwrite=overwrite, runtime_type=runtime_type, **kwargs
+    )
+
+    print("#" * 100)
+
+    if spec_yaml:
+        print(spec_yaml)
+    else:
+        print(output)
