@@ -11,7 +11,7 @@ import json
 
 import yaml
 
-from ads.common.auth import AuthType
+from ads.common.auth import AuthType, ResourcePrincipal
 from ads.opctl import logger
 from ads.opctl.config.base import ConfigProcessor
 from ads.opctl.config.utils import read_from_ini, _DefaultNoneDict
@@ -26,6 +26,7 @@ from ads.opctl.constants import (
     ADS_ML_PIPELINE_CONFIG_FILE_NAME,
     ADS_DATAFLOW_CONFIG_FILE_NAME,
     ADS_LOCAL_BACKEND_CONFIG_FILE_NAME,
+    ADS_MODEL_DEPLOYMENT_CONFIG_FILE_NAME,
     DEFAULT_NOTEBOOK_SESSION_CONDA_DIR,
     BACKEND_NAME,
 )
@@ -42,6 +43,10 @@ class ConfigMerger(ConfigProcessor):
     def process(self, **kwargs) -> None:
         config_string = Template(json.dumps(self.config)).safe_substitute(os.environ)
         self.config = json.loads(config_string)
+
+        if "runtime" not in self.config:
+            self.config["runtime"] = {}
+
         # 1. merge and overwrite values from command line args
         self._merge_config_with_cmd_args(kwargs)
         # 1.5 merge environment variables
@@ -58,7 +63,8 @@ class ConfigMerger(ConfigProcessor):
         # 3. fill in values from default files under ~/.ads_ops
         self._fill_config_with_defaults(ads_config_path)
 
-        logger.debug(f"Config: {self.config}")
+        self._config_flex_shape_details()
+
         return self
 
     def _merge_config_with_cmd_args(self, cmd_args: Dict) -> None:
@@ -109,12 +115,19 @@ class ConfigMerger(ConfigProcessor):
         )
         # set default auth
         if not self.config["execution"].get("auth", None):
-            if is_in_notebook_session():
-                self.config["execution"]["auth"] = AuthType.RESOURCE_PRINCIPAL
+            if ResourcePrincipal.supported():
+                self.config["execution"]["auth"] = (
+                    exec_config.get("auth") or AuthType.RESOURCE_PRINCIPAL
+                )
             else:
-                self.config["execution"]["auth"] = AuthType.API_KEY
+                self.config["execution"]["auth"] = (
+                    exec_config.get("auth") or AuthType.API_KEY
+                )
         # determine profile
-        if self.config["execution"]["auth"] != AuthType.API_KEY:
+        if self.config["execution"]["auth"] in (
+            AuthType.RESOURCE_PRINCIPAL,
+            AuthType.INSTANCE_PRINCIPAL,
+        ):
             profile = self.config["execution"]["auth"].upper()
             exec_config.pop("oci_profile", None)
             self.config["execution"]["oci_profile"] = None
@@ -124,7 +137,6 @@ class ConfigMerger(ConfigProcessor):
             )
             self.config["execution"]["oci_profile"] = profile
         # loading config for corresponding profile
-        logger.info(f"Loading service config for profile {profile}.")
         infra_config = self._get_service_config(profile, ads_config_path)
         if infra_config.get(
             "conda_pack_os_prefix"
@@ -166,11 +178,12 @@ class ConfigMerger(ConfigProcessor):
             return {
                 "oci_config": parser["OCI"].get("oci_config"),
                 "oci_profile": parser["OCI"].get("oci_profile"),
+                "auth": parser["OCI"].get("auth"),
                 "conda_pack_folder": parser["CONDA"].get("conda_pack_folder"),
                 "conda_pack_os_prefix": parser["CONDA"].get("conda_pack_os_prefix"),
             }
         else:
-            logger.info(
+            logger.debug(
                 f"{os.path.join(ads_config_folder, 'config.ini')} does not exist. No config loaded."
             )
             return {}
@@ -178,9 +191,11 @@ class ConfigMerger(ConfigProcessor):
     def _get_service_config(self, oci_profile: str, ads_config_folder: str) -> Dict:
         backend = self.config["execution"].get("backend", None)
         backend_config = {
+            BACKEND_NAME.JOB.value: ADS_JOBS_CONFIG_FILE_NAME,
             BACKEND_NAME.DATAFLOW.value: ADS_DATAFLOW_CONFIG_FILE_NAME,
             BACKEND_NAME.PIPELINE.value: ADS_ML_PIPELINE_CONFIG_FILE_NAME,
             BACKEND_NAME.LOCAL.value: ADS_LOCAL_BACKEND_CONFIG_FILE_NAME,
+            BACKEND_NAME.MODEL_DEPLOYMENT.value: ADS_MODEL_DEPLOYMENT_CONFIG_FILE_NAME,
         }
         config_file = backend_config.get(backend, ADS_JOBS_CONFIG_FILE_NAME)
 
@@ -188,8 +203,57 @@ class ConfigMerger(ConfigProcessor):
             parser = read_from_ini(os.path.join(ads_config_folder, config_file))
             if oci_profile in parser:
                 return parser[oci_profile]
+            if DEFAULT_PROFILE in parser:
+                return parser[DEFAULT_PROFILE]
         else:
-            logger.info(
+            logger.debug(
                 f"{os.path.join(ads_config_folder, config_file)} does not exist. No config loaded."
             )
         return {}
+
+    def _config_flex_shape_details(self):
+        infrastructure = self.config["infrastructure"]
+        backend = self.config["execution"].get("backend", None)
+        if (
+            backend == BACKEND_NAME.JOB.value
+            or backend == BACKEND_NAME.MODEL_DEPLOYMENT.value
+        ):
+            shape_name = infrastructure.get("shape_name", "")
+            if shape_name.endswith(".Flex"):
+                if (
+                    "ocpus" not in infrastructure
+                    or "memory_in_gbs" not in infrastructure
+                ):
+                    raise ValueError(
+                        "Parameters `ocpus` and `memory_in_gbs` must be provided for using flex shape. "
+                        "Call `ads opctl configure` to specify."
+                    )
+                infrastructure["shape_config_details"] = {
+                    "ocpus": infrastructure.pop("ocpus"),
+                    "memory_in_gbs": infrastructure.pop("memory_in_gbs"),
+                }
+        elif backend == BACKEND_NAME.DATAFLOW.value:
+            executor_shape = infrastructure.get("executor_shape", "")
+            driver_shape = infrastructure.get("driver_shape", "")
+            data_flow_shape_config_details = [
+                "driver_shape_memory_in_gbs",
+                "driver_shape_ocpus",
+                "executor_shape_memory_in_gbs",
+                "executor_shape_ocpus",
+            ]
+            # executor_shape and driver_shape must be the same shape family
+            if executor_shape.endswith(".Flex") or driver_shape.endswith(".Flex"):
+                for parameter in data_flow_shape_config_details:
+                    if parameter not in infrastructure:
+                        raise ValueError(
+                            f"Parameters {parameter} must be provided for using flex shape. "
+                            "Call `ads opctl configure` to specify."
+                        )
+                infrastructure["driver_shape_config"] = {
+                    "ocpus": infrastructure.pop("driver_shape_ocpus"),
+                    "memory_in_gbs": infrastructure.pop("driver_shape_memory_in_gbs"),
+                }
+                infrastructure["executor_shape_config"] = {
+                    "ocpus": infrastructure.pop("executor_shape_ocpus"),
+                    "memory_in_gbs": infrastructure.pop("executor_shape_memory_in_gbs"),
+                }
