@@ -1,14 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2021 Oracle and its affiliates.
+# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 import functools
 import os
 
 
-class NotSupportedError(Exception):
+class NotSupportedError(Exception):   # pragma: no cover
     pass
 
 
